@@ -28,6 +28,7 @@ from dotenv import load_dotenv
 import time
 import shutil
 import requests
+import jsonschema  # Added for JSON schema validation
 
 # CLI Version
 VERSION = "1.1.3"
@@ -54,7 +55,7 @@ def show_banner():
 ╚██████╗╚██████╔╝██║ ╚████║   ██║   ███████╗██╔╝ ██╗   ██║   
  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝   ╚═╝   ╚══════╝╚═╝  ╚═╝   ╚═╝   
                                                               
- ██████╗██╗  ██╗ █████╗ ██╗███╗   ██║
+ ██████╗██╗  ██╗ █████╗ ██╗███╗   ██╗
 ██╔════╝██║  ██║██╔══██╗██║████╗  ██║
 ██║     ███████║███████║██║██╔██╗ ██║
 ██║     ██╔══██║██╔══██║██║██║╚██╗██║
@@ -106,6 +107,29 @@ def copy_schema_file(pipeline_id, old_version, new_version):
     else:
         click.secho(f"✗ Source file {source_path} not found for backup.", fg="red", bold=True)
 
+def load_main_schema(main_schema_path):
+    """Load main schema and auto-discover sub-schemas."""
+    try:
+        with open(main_schema_path, 'r') as f:
+            schema = json.load(f)
+        sub_schemas_dir = Path(main_schema_path).parent / "sub-schemas"
+        sub_schemas = []
+        if sub_schemas_dir.exists():
+            for file in sub_schemas_dir.glob("*.json"):
+                try:
+                    with file.open("r") as f:
+                        sub_schema = json.load(f)
+                        if sub_schema.get("metadata", {}).get("parent_pipeline_id") == schema["pipeline_id"]:
+                            sub_schemas.append(str(file))
+                except json.JSONDecodeError:
+                    click.secho(f"✗ Invalid JSON in sub-schema file: {file}", fg="red", bold=True)
+        schema["metadata"]["sub_schemas"] = sub_schemas
+        return schema
+    except json.JSONDecodeError:
+        raise click.ClickException(f"Invalid JSON in main schema: {main_schema_path}")
+    except Exception as e:
+        raise click.ClickException(f"Error loading main schema {main_schema_path}: {str(e)}")
+
 class ColoredGroup(click.Group):
     """Custom Click Group that shows banner and colored help."""
     def format_help(self, ctx, formatter):
@@ -119,7 +143,8 @@ class ColoredGroup(click.Group):
         click.secho("\nCommands (grouped by category):", fg="bright_yellow", bold=True)
 
         click.secho("\n  Initialization:", fg="bright_green", bold=True)
-        click.secho("    init          Initialize a new pipeline with a JSON schema and MongoDB setup.", fg="bright_cyan")
+        click.secho("    init          Initialize a new main pipeline with a JSON schema and MongoDB setup.", fg="bright_cyan")
+        click.secho("    sub-init      Initialize a sub-schema inheriting from main schema.", fg="bright_cyan")
         click.secho("    new-version   Create a new schema version based on semantic versioning (local copy only).", fg="bright_cyan")
 
         click.secho("\n  Schema Management:", fg="bright_green", bold=True)
@@ -194,6 +219,23 @@ def configure_task(i, interactive, allowed_task_types, tasks):
             task["endpoint"] = click.prompt(click.style(f"Task {i+1} LLM endpoint (e.g., services.custom_llm.generate_summary)", fg="bright_blue"), default=task["endpoint"])
         else:
             task["prompt_template"] = click.prompt(click.style(f"Task {i+1} LLM prompt", fg="bright_blue"), default="")
+            output_format = click.prompt(
+                click.style(f"Task {i+1} output format", fg="bright_blue"),
+                default="text",
+                type=click.Choice(["text", "json"]),
+                show_choices=True
+            )
+            task["parameters"]["output_format"] = output_format
+            if output_format == "json":
+                json_schema = click.prompt(
+                    click.style(f"Task {i+1} JSON schema (optional, YAML format)", fg="bright_blue"),
+                    default="{}"
+                )
+                try:
+                    task["parameters"]["json_schema"] = yaml.safe_load(json_schema)
+                except yaml.YAMLError:
+                    click.secho("✗ Invalid JSON schema YAML, using empty schema", fg="red")
+                    task["parameters"]["json_schema"] = {}
     
     elif task_type in ["GET", "POST", "PUT"] and interactive:
         if click.confirm(click.style(f"Add input source or inputs for task {i+1}?", fg="bright_blue"), default=False):
@@ -251,45 +293,33 @@ def cli():
     pass
 
 @cli.command()
-@click.option('--file', type=click.Path(), help='Output path for schema (e.g., schemas/production.json)')
+@click.option('--file', type=click.Path(), default="schemas/production.json", help='Output path for main schema (e.g., schemas/production.json)')
 @click.option('--interactive/--no-interactive', default=True, help='Enable interactive prompts')
 @click.option('--backend-host', default="http://127.0.0.1:8000", help='Backend host URL')
 @click.option('--frontend-host', default="http://127.0.0.1:3636", help='Frontend host URL')
 def init(file, interactive, backend_host, frontend_host):
-    """Initialize a new pipeline with a JSON schema, MongoDB, and ChromaDB setup."""
+    """Initialize a new main pipeline with a JSON schema, MongoDB, and ChromaDB setup."""
     mongod_process = None
     show_banner()
-    click.secho("\nInitializing New Pipeline...", fg="bright_yellow", bold=True)
+    click.secho("\nInitializing Main Pipeline...", fg="bright_yellow", bold=True)
 
-    pipeline_id = click.prompt(click.style("Pipeline ID", fg="bright_blue"), default="new_pipeline") if interactive else "new_pipeline"
-    description = click.prompt(click.style("Description", fg="bright_blue"), default="") if interactive else ""
+    # Create schemas and sub-schemas directories
+    schema_dir = Path(file).parent
+    sub_schemas_dir = schema_dir / "sub-schemas"
+    schema_dir.mkdir(parents=True, exist_ok=True)
+    sub_schemas_dir.mkdir(parents=True, exist_ok=True)
+
+    pipeline_id = click.prompt(click.style("Pipeline ID", fg="bright_blue"), default="production") if interactive else "production"
+    description = click.prompt(click.style("Description", fg="bright_blue"), default="Main orchestrator pipeline") if interactive else "Main orchestrator pipeline"
     created_by = click.prompt(click.style("Creator name", fg="bright_blue"), default="user") if interactive else "user"
 
     if interactive and click.confirm(click.style("Add optional metadata? (tags, pipeline type)", fg="bright_blue"), default=False):
         tags_input = click.prompt(click.style("Tags (comma-separated)", fg="bright_blue"), default="")
         tags = [tag.strip() for tag in tags_input.split(",") if tag.strip()]
-        pipeline_type = click.prompt(click.style("Pipeline type", fg="bright_blue"), default="fullstack-ai")
+        pipeline_type = click.prompt(click.style("Pipeline type", fg="bright_blue"), default="ai-orchestration")
     else:
         tags = []
-        pipeline_type = "fullstack-ai"
-
-    # Add sub-schemas support
-    sub_schemas = []
-    if interactive and click.confirm(click.style("Include sub-schemas in this pipeline?", fg="bright_blue"), default=False):
-        while click.confirm(click.style("Add a sub-schema?", fg="bright_blue"), default=True):
-            sub_schema_path = click.prompt(click.style("Path to sub-schema JSON file", fg="bright_blue"), default="")
-            sub_schema_path = Path(sub_schema_path)
-            if not sub_schema_path.is_file():
-                click.secho(f"✗ Sub-schema file not found: {sub_schema_path}. Please create it first.", fg="red", bold=True)
-                continue
-            try:
-                with sub_schema_path.open("r") as f:
-                    json.load(f)
-                sub_schemas.append(str(sub_schema_path))
-                click.secho(f"✓ Added sub-schema: {sub_schema_path}", fg="bright_green", bold=True)
-            except json.JSONDecodeError:
-                click.secho(f"✗ Invalid JSON in sub-schema file: {sub_schema_path}", fg="red", bold=True)
-                continue
+        pipeline_type = "ai-orchestration"
 
     config = {}
     max_attempts = 3
@@ -408,7 +438,7 @@ def init(file, interactive, backend_host, frontend_host):
                 if not click.confirm(click.style("Add another task?", fg="bright_blue"), default=False):
                     break
         else:
-            json_path = click.prompt(click.style("Path to JSON file for schema", fg="bright_blue"), default=f"schemas/{pipeline_id}/{pipeline_id}.json")
+            json_path = click.prompt(click.style("Path to JSON file for schema", fg="bright_blue"), default=str(Path(file)))
             json_path = Path(json_path)
             if json_path.is_dir():
                 click.secho(f"✗ Provided path is a directory: {json_path}. Please specify a .json file.", fg="red", bold=True)
@@ -463,7 +493,7 @@ def init(file, interactive, backend_host, frontend_host):
             "linked_pipelines": [],
             "status": "draft",
             "parent_version": None,
-            "sub_schemas": sub_schemas
+            "sub_schemas": []
         }
     }
 
@@ -489,34 +519,131 @@ def init(file, interactive, backend_host, frontend_host):
         click.secho(f"✓ Initial schema {pipeline_id} validated and pushed to MongoDB.", fg="bright_green", bold=True)
         update_current_schema(client, db_name, pipeline_id)
     except ValueError as e:
-        click.secho(f"✗ Validation error during push: {e}", fg="red", bold=True)
+        click.secho(f"✗ Validation error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
     except Exception as e:
-        click.secho(f"✗ Push error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Push error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
-    schema_dir = Path(file).parent if file else Path(f"schemas/{pipeline_id}")
-    schema_dir.mkdir(parents=True, exist_ok=True)
-    schema_path = Path(file) if file else schema_dir / f"{pipeline_id}.json"
+    schema_path = Path(file)
     if schema_path.is_dir():
         click.secho(f"✗ Schema path is a directory: {schema_path}. Please specify a .json file.", fg="red", bold=True)
         return
     with schema_path.open("w") as f:
         json.dump(schema, f, indent=2)
     schema_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
-    click.secho(f"✓ Pipeline initialized: {schema_path}", fg="bright_green", bold=True)
+    click.secho(f"✓ Main pipeline initialized: {schema_path}", fg="bright_green", bold=True)
+    click.secho(f"✓ Sub-schemas directory created: {sub_schemas_dir}", fg="bright_green", bold=True)
 
-    # Validate and push sub-schemas
-    for sub_schema_path in sub_schemas:
-        try:
-            with open(sub_schema_path, 'r') as f:
-                sub_schema = json.load(f)
-            sub_schema["metadata"]["parent_pipeline_id"] = pipeline_id
-            validate_schema(sub_schema, is_initial=True)
-            push_schema(client, db_name, sub_schema, is_initial=True)
-            click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} pushed to MongoDB.", fg="bright_green", bold=True)
-        except Exception as e:
-            click.secho(f"✗ Error pushing sub-schema {sub_schema_path}: {e}", fg="red", bold=True)
+@cli.command()
+@click.option('--main-schema', type=click.Path(exists=True), required=True, help='Path to main schema file (e.g., schemas/production.json)')
+@click.option('--interactive/--no-interactive', default=True, help='Enable interactive prompts')
+def sub_init(main_schema, interactive):
+    """Initialize a sub-schema inheriting from the main schema."""
+    show_banner()
+    click.secho("\nInitializing Sub-Schema...", fg="bright_yellow", bold=True)
+
+    # Load main schema to inherit global_config
+    main_schema_data = load_main_schema(main_schema)
+    pipeline_id = click.prompt(click.style("Sub-Schema Pipeline ID", fg="bright_blue"), default="new_sub_pipeline") if interactive else "new_sub_pipeline"
+    description = click.prompt(click.style("Description", fg="bright_blue"), default="Sub-schema pipeline") if interactive else "Sub-schema pipeline"
+    created_by = click.prompt(click.style("Creator name", fg="bright_blue"), default="user") if interactive else "user"
+
+    if interactive and click.confirm(click.style("Add optional metadata? (tags, pipeline type)", fg="bright_blue"), default=False):
+        tags_input = click.prompt(click.style("Tags (comma-separated)", fg="bright_blue"), default="")
+        tags = [tag.strip() for tag in tags_input.split(",") if tag.strip()]
+        pipeline_type = click.prompt(click.style("Pipeline type", fg="bright_blue"), default="ai-query")
+    else:
+        tags = []
+        pipeline_type = "ai-query"
+
+    tasks = []
+    allowed_task_types = main_schema_data["global_config"]["allowed_task_types"]
+    if interactive:
+        config_method = click.prompt(
+            click.style("Configure tasks via (1: CLI, 2: Manual JSON Edit)", fg="bright_blue"),
+            type=click.Choice(["1", "2"]),
+            default="1"
+        )
+        if config_method == "1":
+            while True:
+                tasks = configure_task(len(tasks), interactive, allowed_task_types, tasks)
+                if not click.confirm(click.style("Add another task?", fg="bright_blue"), default=False):
+                    break
+        else:
+            json_path = click.prompt(click.style("Path to JSON file for schema", fg="bright_blue"), default=f"schemas/sub-schemas/{pipeline_id}.json")
+            json_path = Path(json_path)
+            if json_path.is_dir():
+                click.secho(f"✗ Provided path is a directory: {json_path}. Please specify a .json file.", fg="red", bold=True)
+                return
+            json_path.parent.mkdir(parents=True, exist_ok=True)
+            if not json_path.exists():
+                default_tasks = [{"task_id": 1, "description": "Default Task", "task_type": "LOCAL", "endpoint": "contextchain.utils.default_task", "rerun": None}]
+                with json_path.open("w") as f:
+                    json.dump({"tasks": default_tasks}, f, indent=2)
+                click.secho(f"✓ Created default schema file: {json_path}", fg="bright_green", bold=True)
+            try:
+                with json_path.open("r") as f:
+                    tasks_data = json.load(f)
+                tasks = tasks_data.get("tasks", [])
+            except json.JSONDecodeError:
+                click.secho(f"✗ Invalid JSON format in {json_path}", fg="red", bold=True)
+                return
+    else:
+        tasks = [configure_task(0, interactive, allowed_task_types, [])[0]]
+
+    schema = {
+        "pipeline_id": pipeline_id,
+        "schema_version": VERSION,
+        "description": description,
+        "created_by": created_by,
+        "created_at": datetime.utcnow().isoformat() + "Z",
+        "tasks": tasks,
+        "global_config": main_schema_data["global_config"],  # Inherit from main schema
+        "metadata": {
+            "tags": tags,
+            "pipeline_type": pipeline_type,
+            "linked_pipelines": [],
+            "status": "draft",
+            "parent_version": None,
+            "parent_pipeline_id": main_schema_data["pipeline_id"]
+        }
+    }
+
+    if interactive and click.confirm(click.style("Configure advanced settings?", fg="bright_blue"), default=False):
+        logging_level = click.prompt(click.style("Logging level", fg="bright_blue"), default=main_schema_data["global_config"]["logging_level"], type=click.Choice(["INFO", "DEBUG", "WARNING", "ERROR"]))
+        retry_on_failure = click.confirm(click.style("Retry on failure?", fg="bright_blue"), default=main_schema_data["global_config"]["retry_on_failure"])
+        max_retries = click.prompt(click.style("Max retries", fg="bright_blue"), type=int, default=main_schema_data["global_config"]["max_retries"])
+        domains_input = click.prompt(click.style("Allowed domains (comma-separated)", fg="bright_blue"), default=",".join(main_schema_data["global_config"]["allowed_domains"]))
+        allowed_domains = [d.strip() for d in domains_input.split(",") if d.strip()]
+        
+        schema["global_config"].update({
+            "logging_level": logging_level,
+            "retry_on_failure": retry_on_failure,
+            "max_retries": max_retries,
+            "allowed_domains": allowed_domains
+        })
+
+    try:
+        client = get_mongo_client(main_schema_data["global_config"]["default_output_db"])
+        db_name = main_schema_data["global_config"]["default_output_db"]
+        validate_schema(schema, is_initial=True)
+        push_schema(client, db_name, schema, is_initial=True)
+        click.secho(f"✓ Sub-schema {pipeline_id} validated and pushed to MongoDB.", fg="bright_green", bold=True)
+        update_current_schema(client, db_name, pipeline_id)
+    except ValueError as e:
+        click.secho(f"✗ Validation error in sub-schema '{pipeline_id}': {e}", fg="red", bold=True)
+        return
+    except Exception as e:
+        click.secho(f"✗ Push error in sub-schema '{pipeline_id}': {e}", fg="red", bold=True)
+        return
+
+    schema_path = Path(f"schemas/sub-schemas/{pipeline_id}.json")
+    schema_path.parent.mkdir(parents=True, exist_ok=True)
+    with schema_path.open("w") as f:
+        json.dump(schema, f, indent=2)
+    schema_path.chmod(stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
+    click.secho(f"✓ Sub-schema initialized: {schema_path}", fg="bright_green", bold=True)
 
 @cli.command()
 @click.option('--pipeline-id', required=True, help='Pipeline ID')
@@ -588,56 +715,81 @@ def new_version(pipeline_id):
         click.secho(f"✓ New schema version {new_version} created locally: {schema_path}", fg="bright_green", bold=True)
 
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
-@click.option('--file', type=click.Path(exists=True), required=True, help='Path to schema file')
-def schema_compile(file):
-    """Validate a schema file."""
+@click.option('--main-schema', type=click.Path(exists=True), required=True, help='Path to main schema file (e.g., schemas/production.json)')
+def schema_compile(main_schema):
+    """Validate a main schema and its sub-schemas."""
     click.secho("\nValidating Schema...", fg="bright_yellow", bold=True)
     try:
-        with open(file, 'r') as f:
-            schema = json.load(f)
+        schema = load_main_schema(main_schema)
         validate_schema(schema)
-        click.secho("✓ Schema validated successfully.", fg="bright_green", bold=True)
+        click.secho(f"✓ Main schema {schema['pipeline_id']} validated successfully.", fg="bright_green", bold=True)
+        for sub_schema_path in schema["metadata"]["sub_schemas"]:
+            try:
+                with open(sub_schema_path, 'r') as f:
+                    sub_schema = json.load(f)
+                validate_schema(sub_schema)
+                click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} validated successfully.", fg="bright_green", bold=True)
+            except ValueError as e:
+                click.secho(f"✗ Validation error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
+                sys.exit(1)
+            except Exception as e:
+                click.secho(f"✗ Error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
+                sys.exit(1)
     except ValueError as e:
-        click.secho(f"✗ Validation error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Validation error in main schema '{schema['pipeline_id']}': {e}", fg="red", bold=True)
         sys.exit(1)
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in main schema '{schema['pipeline_id']}': {e}", fg="red", bold=True)
         sys.exit(1)
 
 @cli.command()
-@click.option('--file', type=click.Path(exists=True), required=True, help='Path to schema file')
-def schema_push(file):
-    """Push a schema to MongoDB with versioning."""
+@click.option('--main-schema', type=click.Path(exists=True), required=True, help='Path to main schema file (e.g., schemas/production.json)')
+def schema_push(main_schema):
+    """Push a main schema and its sub-schemas to MongoDB with versioning."""
     click.secho("\nPushing Schema to MongoDB...", fg="bright_yellow", bold=True)
     try:
         config_path = Path("config/default_config.yaml")
         with config_path.open("r") as f:
             config = yaml.safe_load(f)
-        with open(file, 'r') as f:
-            schema = json.load(f)
         client = get_mongo_client(config["uri"])
         db_name = config["db_name"]
+        schema = load_main_schema(main_schema)
         validate_schema(schema)
         push_schema(client, db_name, schema)
-        click.secho(f"✓ Schema {schema['pipeline_id']} pushed to MongoDB with version {schema['schema_version']}.", fg="bright_green", bold=True)
+        click.secho(f"✓ Main schema {schema['pipeline_id']} pushed to MongoDB with version {schema['schema_version']}.", fg="bright_green", bold=True)
         update_current_schema(client, db_name, schema["pipeline_id"])
         copy_schema_file(schema["pipeline_id"], schema["metadata"].get("parent_version", "v0.0.0"), schema["schema_version"])
+        for sub_schema_path in schema["metadata"]["sub_schemas"]:
+            try:
+                with open(sub_schema_path, 'r') as f:
+                    sub_schema = json.load(f)
+                validate_schema(sub_schema)
+                push_schema(client, db_name, sub_schema)
+                click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} pushed to MongoDB with version {sub_schema['schema_version']}.", fg="bright_green", bold=True)
+                update_current_schema(client, db_name, sub_schema["pipeline_id"])
+                copy_schema_file(sub_schema["pipeline_id"], sub_schema["metadata"].get("parent_version", "v0.0.0"), sub_schema["schema_version"])
+            except ValueError as e:
+                click.secho(f"✗ Validation error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
+                sys.exit(1)
+            except Exception as e:
+                click.secho(f"✗ Push error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
+                sys.exit(1)
     except ValueError as e:
-        click.secho(f"✗ Validation error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Validation error in main schema '{schema['pipeline_id']}': {e}", fg="red", bold=True)
         sys.exit(1)
     except Exception as e:
-        click.secho(f"✗ Push error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Push error in main schema '{schema['pipeline_id']}': {e}", fg="red", bold=True)
         sys.exit(1)
 
 @cli.command()
 @click.option('--pipeline_id', required=True, help='Pipeline ID')
 @click.option('--version', help='Schema version (default: latest)')
 def run(pipeline_id, version):
-    """Run an entire pipeline."""
+    """Run an entire pipeline, including sub-schemas."""
     click.secho(f"\nRunning Pipeline {pipeline_id}...", fg="bright_yellow", bold=True)
     try:
         config_path = Path("config/default_config.yaml")
@@ -650,9 +802,18 @@ def run(pipeline_id, version):
             click.secho(f"✗ Pipeline {pipeline_id} not found.", fg="red", bold=True)
             sys.exit(1)
         execute_pipeline(client, db_name, schema)
-        click.secho(f"✓ Pipeline {pipeline_id} executed successfully.", fg="bright_green", bold=True)
+        for sub_schema_path in schema["metadata"]["sub_schemas"]:
+            try:
+                with open(sub_schema_path, 'r') as f:
+                    sub_schema = json.load(f)
+                execute_pipeline(client, db_name, sub_schema)
+                click.secho(f"✓ Sub-schema {sub_schema['pipeline_id']} executed successfully.", fg="bright_green", bold=True)
+            except Exception as e:
+                click.secho(f"✗ Execution error in sub-schema '{sub_schema['pipeline_id']}' ({sub_schema_path}): {e}", fg="red", bold=True)
+                sys.exit(1)
+        click.secho(f"✓ Pipeline {pipeline_id} and sub-schemas executed successfully.", fg="bright_green", bold=True)
     except Exception as e:
-        click.secho(f"✗ Execution error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Execution error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         sys.exit(1)
 
 @cli.command()
@@ -674,12 +835,20 @@ def run_task(pipeline_id, task_id, version):
             sys.exit(1)
         task = next((t for t in schema["tasks"] if t["task_id"] == task_id), None)
         if not task:
-            click.secho(f"✗ Task {task_id} not found.", fg="red", bold=True)
+            click.secho(f"✗ Task {task_id} not found in pipeline '{pipeline_id}'.", fg="red", bold=True)
             sys.exit(1)
-        execute_single_task(client, db_name, schema, task)
-        click.secho(f"✓ Task {task_id} executed successfully.", fg="bright_green", bold=True)
+        if task["task_type"] == "LLM_GENERATE" and task["parameters"].get("output_format") == "json":
+            result = execute_single_task(client, db_name, schema, task)
+            try:
+                jsonschema.validate(result, task["parameters"].get("json_schema", {}))
+            except jsonschema.ValidationError as e:
+                click.secho(f"✗ JSON schema validation failed for task '{task_id}' in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
+                sys.exit(1)
+        else:
+            execute_single_task(client, db_name, schema, task)
+        click.secho(f"✓ Task {task_id} in pipeline '{pipeline_id}' executed successfully.", fg="bright_green", bold=True)
     except Exception as e:
-        click.secho(f"✗ Execution error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Execution error in task '{task_id}' in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         sys.exit(1)
 
 @cli.command()
@@ -702,7 +871,7 @@ def version_list(pipeline_id):
             is_latest = " (latest)" if v.get("is_latest", False) else ""
             click.secho(f"  • Version {v['schema_version']}{is_latest}: Created {v['created_at']}", fg="bright_cyan")
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
@@ -721,10 +890,10 @@ def version_rollback(pipeline_id, version):
         click.secho(f"✓ Rolled back {pipeline_id} to version {version}.", fg="bright_green", bold=True)
         update_current_schema(client, db_name, pipeline_id, version)
     except ValueError as e:
-        click.secho(f"✗ Rollback error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Rollback error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
@@ -807,7 +976,7 @@ def logs(pipeline_id):
         else:
             click.secho(f"No logs found for {pipeline_id}.", fg="bright_yellow")
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
@@ -829,7 +998,7 @@ def results(task_id):
         else:
             click.secho(f"No results found for task {task_id}.", fg="bright_yellow")
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in task '{task_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
@@ -856,7 +1025,7 @@ def schema_pull(pipeline_id, version):
             click.secho(f"✗ Schema {pipeline_id} not found.", fg="red", bold=True)
             return
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
         return
 
 @cli.command()
@@ -964,7 +1133,7 @@ def metrics(pipeline_id):
         else:
             click.secho(f"No metrics found for {pipeline_id}.", fg="bright_yellow")
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
 
 @cli.command()
 @click.option('--task_id', type=int, required=True, help='Task ID')
@@ -981,7 +1150,7 @@ def evaluate(task_id):
         evaluation = evaluate_results(results)
         click.secho(f"Evaluation: {evaluation}", fg="bright_green", bold=True)
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in task '{task_id}': {e}", fg="red", bold=True)
 
 @cli.command()
 @click.option('--pipeline_id', required=True, help='Pipeline ID')
@@ -1002,7 +1171,7 @@ def schema_current(pipeline_id):
         else:
             click.secho(f"✗ Schema {pipeline_id} not found.", fg="red", bold=True)
     except Exception as e:
-        click.secho(f"✗ Error: {e}", fg="red", bold=True)
+        click.secho(f"✗ Error in pipeline '{pipeline_id}': {e}", fg="red", bold=True)
 
 @cli.group()
 def plugin():
